@@ -28,13 +28,13 @@
  *
  */
 
-#include "util/omc_error.h"
-#include "simulation_data.h"
-#include "util/rtclock.h"
+#include "../util/omc_error.h"
+#include "../simulation_data.h"
+#include "../util/rtclock.h"
 #include "modelinfo.h"
 #include "simulation_info_json.h"
 #include "simulation_runtime.h"
-#include "util/omc_mmap.h"
+#include "../util/omc_mmap.h"
 #include "solver/model_help.h"
 
 
@@ -44,7 +44,8 @@
 #include <time.h>
 #include <assert.h>
 #include <stdint.h>
-#include "util/read_matlab4.h"
+#include "../util/read_matlab4.h"
+#include "../util/omc_file.h"
 
 /* UNDEF to debug the gnuplot file */
 #define NO_PIPE
@@ -52,7 +53,7 @@
 /* Returns -1 if the file was not found */
 static size_t fileSize(const char *filename) {
   size_t sz = -1;
-  FILE *f = fopen(filename, "rb");
+  FILE *f = omc_fopen(filename, "rb");
   if(f) {
     fseek(f, 0, SEEK_END);
     sz = ftell(f);
@@ -195,7 +196,7 @@ static void printPlotCommand(FILE *plt, const char *plotFormat, const char *titl
     }
     fprintf(plt, "set xlabel \"Global step number\"\n");
     fprintf(plt, "set ylabel \"Execution count\"\n");
-    fprintf(plt, "set output \"%s_prof.%s%d_count.%s\"\n", prefix, idPrefix, id, plotFormat);
+    fprintf(plt, "set output \"%s%s_prof.%s%d_count.%s\"\n", outputPath, prefix, idPrefix, id, plotFormat);
     fprintf(plt, formatCount, outputPath, prefix, 1+numFnsAndBlocks, 2+i, 2);
   }
 }
@@ -235,7 +236,6 @@ static void printVar(FILE *fout, int level, VAR_INFO* info) {
   fprintf(fout, "</variable>\n");
 }
 
-
 static void printFunctions(FILE *fout, FILE *plt, const char *plotFormat, const char *outputPath, const char *modelFilePrefix, DATA *data) {
   int i;
   for(i=0; i<data->modelData->modelDataXml.nFunctions; i++) {
@@ -269,13 +269,52 @@ static void printProfileBlocks(FILE *fout, FILE *plt, const char *plotFormat, co
   }
 }
 
-static void printEquations(FILE *fout, int n, MODEL_DATA_XML *xml) {
+
+static int getVarIdByName(DATA *data, const char* varName)
+{
+  int i;
+  for(i=0;i<data->modelData->nVariablesReal;++i)
+    if (strcmp(varName, data->modelData->realVarsData[i].info.name) == 0)
+      return data->modelData->realVarsData[i].info.id;
+
+  for(i=0;i<data->modelData->nParametersReal;++i)
+    if (strcmp(varName, data->modelData->realParameterData[i].info.name) == 0)
+      return data->modelData->realParameterData[i].info.id;
+
+  for(i=0;i<data->modelData->nVariablesInteger;++i)
+    if (strcmp(varName, data->modelData->integerVarsData[i].info.name) == 0)
+      return data->modelData->integerVarsData[i].info.id;
+
+  for(i=0;i<data->modelData->nParametersInteger;++i)
+    if (strcmp(varName, data->modelData->integerParameterData[i].info.name) == 0)
+      return data->modelData->integerParameterData[i].info.id;
+
+  for(i=0;i<data->modelData->nVariablesBoolean;++i)
+    if (strcmp(varName, data->modelData->booleanVarsData[i].info.name) == 0)
+      return data->modelData->booleanVarsData[i].info.id;
+
+  for(i=0;i<data->modelData->nParametersBoolean;++i)
+    if (strcmp(varName, data->modelData->booleanParameterData[i].info.name) == 0)
+      return data->modelData->booleanParameterData[i].info.id;
+
+  for(i=0;i<data->modelData->nVariablesString;++i)
+    if (strcmp(varName, data->modelData->stringVarsData[i].info.name) == 0)
+      return data->modelData->stringVarsData[i].info.id;
+
+  for(i=0;i<data->modelData->nParametersString;++i)
+    if (strcmp(varName, data->modelData->stringParameterData[i].info.name) == 0)
+      return data->modelData->stringParameterData[i].info.id;
+
+  return 0;
+}
+
+static void printEquations(FILE *fout, int n, MODEL_DATA_XML *xml, DATA *data) {
   int i,j;
   for(i=0; i<n; i++) {
     indent(fout,2);fprintf(fout, "<equation id=\"eq%d\">\n", modelInfoGetEquation(xml,i).id);
     indent(fout,4);fprintf(fout, "<refs>\n");
     for(j=0; j<modelInfoGetEquation(xml,i).numVar; j++) {
-      indent(fout,6);fprintf(fout, "<ref refid=\"var%d\" />\n", 0 /* modelInfoGetEquation(xml,i).vars[j]->id */);
+      indent(fout,6);fprintf(fout, "<ref refid=\"var%d\" />\n", getVarIdByName(data, modelInfoGetEquation(xml,i).vars[j]));
     }
     indent(fout,4);fprintf(fout, "</refs>\n");
     indent(fout,4);fprintf(fout, "<calcinfo time=\"%f\" count=\"%lu\"/>\n", rt_accumulated(SIM_TIMER_FIRST_FUNCTION + xml->nFunctions + xml->nProfileBlocks + modelInfoGetEquation(xml,i).id), (long) rt_ncall(SIM_TIMER_FIRST_FUNCTION + xml->nFunctions + xml->nProfileBlocks + modelInfoGetEquation(xml,i).id));
@@ -323,7 +362,7 @@ int printModelInfo(DATA *data, threadData_t *threadData, const char *outputPath,
   if (0 > GC_asprintf(&fullFileName, "%s%s", outputPath, filename)) {
     throwStreamPrint(NULL, "modelinfo.c: Error: can not allocate memory.");
   }
-  FILE *fout = fopen(fullFileName, "w");
+  FILE *fout = omc_fopen(fullFileName, "w");
   FILE *plotCommands;
   time_t t;
   int i;
@@ -332,7 +371,7 @@ int printModelInfo(DATA *data, threadData_t *threadData, const char *outputPath,
   if (0 > GC_asprintf(&fullPlotFile, "%s%s", outputPath, plotfile)) {
     throwStreamPrint(NULL, "modelinfo.c: Error: can not allocate memory.");
   }
-  plotCommands = fopen(fullPlotFile, "w");
+  plotCommands = omc_fopen(fullPlotFile, "w");
 #else
   plotCommands = popen("gnuplot", "w");
 #endif
@@ -439,7 +478,7 @@ int printModelInfo(DATA *data, threadData_t *threadData, const char *outputPath,
   fprintf(fout, "</functions>\n");
 
   fprintf(fout, "<equations>\n");
-  printEquations(fout, data->modelData->modelDataXml.nEquations, &data->modelData->modelDataXml);
+  printEquations(fout, data->modelData->modelDataXml.nEquations, &data->modelData->modelDataXml, data);
   fprintf(fout, "</equations>\n");
 
   fprintf(fout, "<profileblocks>\n");
@@ -453,12 +492,12 @@ int printModelInfo(DATA *data, threadData_t *threadData, const char *outputPath,
     const char *omhome = data->simulationInfo->OPENMODELICAHOME;
     char *buf = NULL;
     int genHtmlRes;
-    buf = (char*)malloc(230 + 2*strlen(plotfile) + 2*(omhome ? strlen(omhome) : 0));
+    buf = (char*)malloc(230 + 4 + 2*strlen(plotfile) + 2*strlen(outputPath) + 2*(omhome ? strlen(omhome) : 0));
     assert(buf);
 #if defined(__MINGW32__) || defined(_MSC_VER) || defined(NO_PIPE)
     if(omhome) {
 #if defined(__MINGW32__) || defined(_MSC_VER)
-      sprintf(buf, "%s/lib/omc/libexec/gnuplot/binary/gnuplot.exe %s%s", omhome, outputPath, plotfile);
+      sprintf(buf, "\"\"%s/lib/omc/libexec/gnuplot/binary/gnuplot.exe\" %s%s\"", omhome, outputPath, plotfile);
 #else
       sprintf(buf, "gnuplot %s", plotfile);
 #endif
@@ -481,7 +520,13 @@ int printModelInfo(DATA *data, threadData_t *threadData, const char *outputPath,
 #else
       const char *xsltproc = "xsltproc";
 #endif
+
+#if defined(__MINGW32__) || defined(_MSC_VER)
+      sprintf(buf, "\"\"%s\" -o %s%s_prof.html \"%s/share/omc/scripts/default_profiling.xsl\" %s%s_prof.xml\"", xsltproc, outputPath, data->modelData->modelFilePrefix, omhome, outputPath, data->modelData->modelFilePrefix);
+#else
       sprintf(buf, "%s -o %s%s_prof.html %s/share/omc/scripts/default_profiling.xsl %s%s_prof.xml", xsltproc, outputPath, data->modelData->modelFilePrefix, omhome, outputPath, data->modelData->modelFilePrefix);
+#endif
+
 #if defined(__MINGW32__) || defined(_MSC_VER)
       free(xsltproc);
 #endif
@@ -566,7 +611,7 @@ int printModelInfoJSON(DATA *data, threadData_t *threadData, const char *outputP
   if (0 > GC_asprintf(&fullFileName, "%s%s", outputPath, filename)) {
     throwStreamPrint(NULL, "modelinfo.c: Error: can not allocate memory.");
   }
-  FILE *fout = fopen(fullFileName, "wb");
+  FILE *fout = omc_fopen(fullFileName, "wb");
   time_t t;
   long i;
   double totalTimeEqs = 0;
