@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -39,18 +39,23 @@
 #include "initialization/initialization.h"
 #include "nonlinearSystem.h"
 #include "newtonIteration.h"
+#include "cvode_solver.h"
 #include "dassl.h"
 #include "ida_solver.h"
 #include "delay.h"
 #include "events.h"
+#include "util/parallel_helper.h"
 #include "util/varinfo.h"
 #include "radau.h"
 #include "model_help.h"
 #include "meta/meta_modelica.h"
 #include "simulation/solver/epsilon.h"
 #include "simulation/solver/external_input.h"
+#include "synchronous.h"
 #include "linearSystem.h"
 #include "sym_solver_ssc.h"
+#include "gbode_main.h"
+#include "gbode_util.h"
 #include "irksco.h"
 #if !defined(OMC_MINIMAL_RUNTIME)
 #include "simulation/solver/embedded_server.h"
@@ -90,7 +95,7 @@ static int sym_solver_step(DATA* data, threadData_t *threadData, SOLVER_INFO* so
 
 static int radau_lobatto_step(DATA* data, SOLVER_INFO* solverInfo);
 
-#ifdef WITH_IPOPT
+#ifdef OMC_HAVE_IPOPT
 static int ipopt_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo);
 #endif
 
@@ -126,7 +131,7 @@ int solver_main_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverIn
     return retVal;
 #endif
 
-#ifdef WITH_IPOPT
+#ifdef OMC_HAVE_IPOPT
   case S_OPTIMIZATION:
     if ((int)(data->modelData->nStates + data->modelData->nInputVars) > 0){
       retVal = ipopt_step(data, threadData, solverInfo);
@@ -154,6 +159,12 @@ int solver_main_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverIn
       data->simulationInfo->solverSteps = solverInfo->solverStats[0] + solverInfo->solverStatsTmp[0];
     TRACE_POP
     return retVal;
+  case S_CVODE:
+    retVal = cvode_solver_step(data, threadData, solverInfo);
+    if(omc_flag[FLAG_SOLVER_STEPS])
+      data->simulationInfo->solverSteps = solverInfo->solverStats[0] + solverInfo->solverStatsTmp[0];
+    TRACE_POP
+    return retVal;
 #endif
   case S_ERKSSC:
     retVal = rungekutta_step_ssc(data, threadData, solverInfo);
@@ -172,15 +183,18 @@ int solver_main_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverIn
       data->simulationInfo->solverSteps = solverInfo->solverStats[0] + solverInfo->solverStatsTmp[0];
     return retVal;
   case S_IRKSCO:
-  {
     retVal = irksco_midpoint_rule(data, threadData, solverInfo);
     if(omc_flag[FLAG_SOLVER_STEPS])
       data->simulationInfo->solverSteps = solverInfo->solverStats[0] + solverInfo->solverStatsTmp[0];
     return retVal;
+  case S_GBODE:
+    retVal = gbode_main(data, threadData, solverInfo);
+    if(omc_flag[FLAG_SOLVER_STEPS])
+      data->simulationInfo->solverSteps = solverInfo->solverStats[0] + solverInfo->solverStatsTmp[0];
+    return retVal;
+  default:
+    throwStreamPrint(threadData, "Unhandled case in solver_main_step.");
   }
-
-  }
-
   TRACE_POP
   return 1;
 }
@@ -224,14 +238,14 @@ int initializeSolverData(DATA* data, threadData_t *threadData, SOLVER_INFO* solv
     solverInfo->integratorSteps = 0;
   }
 
-  /* set tolerance for ZeroCrossings */
-  /*  TODO: Check this! */
-  /*  setZCtol(fmin(simInfo->stepSize, simInfo->tolerance)); */
+  /* Deprecation warnings */
+  deprecationWarningGBODE(solverInfo->solverMethod);
 
   switch (solverInfo->solverMethod)
   {
   case S_SYM_SOLVER:
-  case S_EULER: break;
+  case S_EULER:
+  case S_QSS: break;
   case S_SYM_SOLVER_SSC:
   {
     allocateSymSolverSsc(solverInfo, data->modelData->nStates);
@@ -239,7 +253,14 @@ int initializeSolverData(DATA* data, threadData_t *threadData, SOLVER_INFO* solv
   }
   case S_IRKSCO:
   {
-    allocateIrksco(solverInfo, data->modelData->nStates, data->modelData->nZeroCrossings);
+    allocateIrksco(data, threadData, solverInfo, data->modelData->nStates, data->modelData->nZeroCrossings);
+    break;
+  }
+  case S_GBODE:
+  {
+    if (gbode_allocateData(data, threadData, solverInfo) != 0) {
+      throwStreamPrint(threadData, "Failed to allocate memory for generic multigrid solver.");
+    }
     break;
   }
   case S_ERKSSC:
@@ -280,7 +301,6 @@ int initializeSolverData(DATA* data, threadData_t *threadData, SOLVER_INFO* solv
     solverInfo->solverData = rungeData;
     break;
   }
-  case S_QSS: break;
 #if !defined(OMC_MINIMAL_RUNTIME)
   case S_DASSL:
   {
@@ -291,7 +311,7 @@ int initializeSolverData(DATA* data, threadData_t *threadData, SOLVER_INFO* solv
     break;
   }
 #endif
-#ifdef WITH_IPOPT
+#ifdef OMC_HAVE_IPOPT
   case S_OPTIMIZATION:
   {
     infoStreamPrint(LOG_SOLVER, 0, "Initializing optimizer");
@@ -337,6 +357,16 @@ int initializeSolverData(DATA* data, threadData_t *threadData, SOLVER_INFO* solv
     solverInfo->solverData = idaData;
     break;
   }
+  case S_CVODE:
+  {
+    CVODE_SOLVER* cvodeData = NULL;
+    infoStreamPrint(LOG_SOLVER, 0, "Initializing CVODE ODE Solver");
+    cvodeData = (CVODE_SOLVER*) calloc(1, sizeof(CVODE_SOLVER));
+    assertStreamPrint(threadData, cvodeData != NULL, "Out of memory");
+    retValue = cvode_solver_initial(data, threadData, solverInfo, cvodeData, 0 /* not FMI */);
+    solverInfo->solverData = cvodeData;
+    break;
+  }
 #endif
   default:
     errorStreamPrint(LOG_SOLVER, 0, "Solver %s disabled on this configuration", SOLVER_METHOD_NAME[solverInfo->solverMethod]);
@@ -360,57 +390,64 @@ int freeSolverData(DATA* data, SOLVER_INFO* solverInfo)
   int retValue = 0;
   int i;
 
+  freeList(solverInfo->eventLst);
   /* free solver statistics */
   free(solverInfo->solverStats);
   free(solverInfo->solverStatsTmp);
   /* deintialize solver related workspace */
-  if (solverInfo->solverMethod == S_SYM_SOLVER_SSC)
+  switch (solverInfo->solverMethod)
   {
+  case S_EULER:
+  case S_SYM_SOLVER:
+  case S_QSS: break;
+  case S_SYM_SOLVER_SSC:
     freeSymSolverSsc(solverInfo);
-  }
-  else if(solverInfo->solverMethod == S_RUNGEKUTTA || solverInfo->solverMethod == S_HEUN
-         || solverInfo->solverMethod == S_ERKSSC)
-  {
+    break;
+  case S_RUNGEKUTTA:
+  case S_HEUN:
+  case S_ERKSSC:
     /* free RK work arrays */
     for(i = 0; i < ((RK4_DATA*)(solverInfo->solverData))->work_states_ndims + 1; i++)
       free(((RK4_DATA*)(solverInfo->solverData))->work_states[i]);
     free(((RK4_DATA*)(solverInfo->solverData))->work_states);
     free((RK4_DATA*)solverInfo->solverData);
-  }
-  else if (solverInfo->solverMethod == S_IRKSCO)
-  {
+    break;
+  case S_IRKSCO:
     freeIrksco(solverInfo);
-  }
+    break;
+  case S_GBODE:
+    gbode_freeData(data, solverInfo->solverData);
+    break;
 #if !defined(OMC_MINIMAL_RUNTIME)
-  else if(solverInfo->solverMethod == S_DASSL)
-  {
+  case S_DASSL:
     /* De-Initial DASSL solver */
-    dassl_deinitial(solverInfo->solverData);
-  }
+    dassl_deinitial(data, solverInfo->solverData);
+    break;
 #endif
-#ifdef WITH_IPOPT
-  else if(solverInfo->solverMethod == S_OPTIMIZATION)
-  {
+#ifdef OMC_HAVE_IPOPT
+  case S_OPTIMIZATION:
     /* free  work arrays */
     /*destroyIpopt(solverInfo);*/
-  }
+    break;
 #endif
 #ifdef WITH_SUNDIALS
-  else if(solverInfo->solverMethod == S_IMPEULER ||
-          solverInfo->solverMethod == S_TRAPEZOID ||
-          solverInfo->solverMethod == S_IMPRUNGEKUTTA)
-  {
-    /* free  work arrays */
-    freeKinOde(data, solverInfo);
-  }
-  else if(solverInfo->solverMethod == S_IDA)
-  {
+  case S_IMPEULER:
+  case S_TRAPEZOID:
+  case S_IMPRUNGEKUTTA:
     /* free  work arrays */
+    freeKinOde((KINODE*)solverInfo->solverData);
+    break;
+  case S_IDA:
+    /* free work arrays */
     ida_solver_deinitial(solverInfo->solverData);
-  }
+    break;
+  case S_CVODE:
+    /* free work arrays */
+    cvode_solver_deinitial(solverInfo->solverData);
+    break;
 #endif
-  {
-    /* free other solver memory */
+  default:
+    throwStreamPrint(NULL, "Unknown solver %u encountered. Possibly leaking memory!", solverInfo->solverMethod);
   }
 
   return retValue;
@@ -432,6 +469,7 @@ int initializeModel(DATA* data, threadData_t *threadData, const char* init_initM
 {
   TRACE_PUSH
   int retValue = 0;
+  int usedLocal = 0;
 
   SIMULATION_INFO *simInfo = data->simulationInfo;
 
@@ -439,7 +477,6 @@ int initializeModel(DATA* data, threadData_t *threadData, const char* init_initM
   {
     rt_accumulate(SIM_TIMER_PREINIT);
     rt_tick(SIM_TIMER_INIT);
-    rt_tick(SIM_TIMER_TOTAL);
   }
 
   copyStartValuestoInitValues(data);
@@ -465,6 +502,16 @@ int initializeModel(DATA* data, threadData_t *threadData, const char* init_initM
       simInfo->stopTime = simInfo->startTime;
       retValue = -1;
     }
+    if (!retValue)
+    {
+      if (data->simulationInfo->homotopySteps == 0) {
+        infoStreamPrint(LOG_SUCCESS, 0, "The initialization finished successfully without homotopy method.");
+      }
+      else {
+        usedLocal = data->callback->useHomotopy == 0 || data->callback->useHomotopy == 3;
+        infoStreamPrint(LOG_SUCCESS, 0, "The initialization finished successfully with %d %shomotopy steps.", data->simulationInfo->homotopySteps, usedLocal? "local ":"");
+      }
+    }
 
     success = 1;
 #if !defined(OMC_EMCC)
@@ -506,6 +553,7 @@ int finishSimulation(DATA* data, threadData_t *threadData, SOLVER_INFO* solverIn
 
   int retValue = 0;
   int ui;
+  double t, total100;
 
   SIMULATION_INFO *simInfo = data->simulationInfo;
 
@@ -539,23 +587,23 @@ int finishSimulation(DATA* data, threadData_t *threadData, SOLVER_INFO* solverIn
 
     infoStreamPrint(LOG_STATS, 1, "### STATISTICS ###");
 
+    total100 = rt_accumulated(SIM_TIMER_TOTAL)/100.0;
+
     infoStreamPrint(LOG_STATS, 1, "timer");
     infoStreamPrint(LOG_STATS, 0, "%12gs          reading init.xml", rt_accumulated(SIM_TIMER_INIT_XML));
     infoStreamPrint(LOG_STATS, 0, "%12gs          reading info.xml", rt_accumulated(SIM_TIMER_INFO_XML));
-    infoStreamPrint(LOG_STATS, 0, "%12gs          pre-initialization", rt_accumulated(SIM_TIMER_PREINIT));
-    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] initialization", rt_accumulated(SIM_TIMER_INIT), rt_accumulated(SIM_TIMER_INIT)/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
-    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] steps", rt_accumulated(SIM_TIMER_STEP), rt_accumulated(SIM_TIMER_STEP)/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
-    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] creating output-file", rt_accumulated(SIM_TIMER_OUTPUT), rt_accumulated(SIM_TIMER_OUTPUT)/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
-    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] event-handling", rt_accumulated(SIM_TIMER_EVENT), rt_accumulated(SIM_TIMER_EVENT)/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
-    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] jacobian calculation", rt_accumulated(SIM_TIMER_JACOBIAN), rt_accumulated(SIM_TIMER_JACOBIAN)/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
-    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] overhead", rt_accumulated(SIM_TIMER_OVERHEAD), rt_accumulated(SIM_TIMER_OVERHEAD)/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
+    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] pre-initialization", rt_accumulated(SIM_TIMER_PREINIT), rt_accumulated(SIM_TIMER_PREINIT)/total100);
+    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] initialization", rt_accumulated(SIM_TIMER_INIT), rt_accumulated(SIM_TIMER_INIT)/total100);
+    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] steps", rt_accumulated(SIM_TIMER_STEP), rt_accumulated(SIM_TIMER_STEP)/total100);
+    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] solver (excl. callbacks)", rt_accumulated(SIM_TIMER_SOLVER), rt_accumulated(SIM_TIMER_SOLVER)/total100);
+    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] creating output-file", rt_accumulated(SIM_TIMER_OUTPUT), rt_accumulated(SIM_TIMER_OUTPUT)/total100);
+    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] event-handling", rt_accumulated(SIM_TIMER_EVENT), rt_accumulated(SIM_TIMER_EVENT)/total100);
+    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] overhead", rt_accumulated(SIM_TIMER_OVERHEAD), rt_accumulated(SIM_TIMER_OVERHEAD)/total100);
 
-    if(S_OPTIMIZATION != solverInfo->solverMethod)
-      infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] simulation", rt_accumulated(SIM_TIMER_TOTAL)-rt_accumulated(SIM_TIMER_OVERHEAD)-rt_accumulated(SIM_TIMER_EVENT)-rt_accumulated(SIM_TIMER_OUTPUT)-rt_accumulated(SIM_TIMER_STEP)-rt_accumulated(SIM_TIMER_INIT)-rt_accumulated(SIM_TIMER_PREINIT), (rt_accumulated(SIM_TIMER_TOTAL)-rt_accumulated(SIM_TIMER_OVERHEAD)-rt_accumulated(SIM_TIMER_EVENT)-rt_accumulated(SIM_TIMER_OUTPUT)-rt_accumulated(SIM_TIMER_STEP)-rt_accumulated(SIM_TIMER_INIT)-rt_accumulated(SIM_TIMER_PREINIT))/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
-    else
-      infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] optimization", rt_accumulated(SIM_TIMER_TOTAL)-rt_accumulated(SIM_TIMER_OVERHEAD)-rt_accumulated(SIM_TIMER_EVENT)-rt_accumulated(SIM_TIMER_OUTPUT)-rt_accumulated(SIM_TIMER_STEP)-rt_accumulated(SIM_TIMER_INIT)-rt_accumulated(SIM_TIMER_PREINIT), (rt_accumulated(SIM_TIMER_TOTAL)-rt_accumulated(SIM_TIMER_OVERHEAD)-rt_accumulated(SIM_TIMER_EVENT)-rt_accumulated(SIM_TIMER_OUTPUT)-rt_accumulated(SIM_TIMER_STEP)-rt_accumulated(SIM_TIMER_INIT)-rt_accumulated(SIM_TIMER_PREINIT))/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
+    t = rt_accumulated(SIM_TIMER_TOTAL)-rt_accumulated(SIM_TIMER_OVERHEAD)-rt_accumulated(SIM_TIMER_EVENT)-rt_accumulated(SIM_TIMER_OUTPUT)-rt_accumulated(SIM_TIMER_STEP)-rt_accumulated(SIM_TIMER_INIT)-rt_accumulated(SIM_TIMER_PREINIT)-rt_accumulated(SIM_TIMER_SOLVER);
+    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] %s", t, t/total100, S_OPTIMIZATION == solverInfo->solverMethod ? "optimization" : "simulation");
 
-    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] total", rt_accumulated(SIM_TIMER_TOTAL), rt_accumulated(SIM_TIMER_TOTAL)/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
+    infoStreamPrint(LOG_STATS, 0, "%12gs [100.0%%] total", rt_accumulated(SIM_TIMER_TOTAL));
     messageClose(LOG_STATS);
 
     infoStreamPrint(LOG_STATS, 1, "events");
@@ -579,22 +627,54 @@ int finishSimulation(DATA* data, threadData_t *threadData, SOLVER_INFO* solverIn
       infoStreamPrint(LOG_STATS, 0, "%5d evaluations of jacobian", solverInfo->solverStats[2]);
       infoStreamPrint(LOG_STATS, 0, "%5d error test failures", solverInfo->solverStats[3]);
       infoStreamPrint(LOG_STATS, 0, "%5d convergence test failures", solverInfo->solverStats[4]);
+      infoStreamPrint(LOG_STATS, 0, "%gs time of jacobian evaluation", rt_accumulated(SIM_TIMER_JACOBIAN));
+#ifdef USE_PARJAC
+      infoStreamPrint(LOG_STATS, 0, "%i OpenMP-threads used for jacobian evaluation", omc_get_max_threads());
+      int chunk_size;
+      omp_sched_t kind;
+      omp_get_schedule(&kind, &chunk_size);
+      infoStreamPrint(LOG_STATS, 0, "Schedule: %i Chunk Size: %i", kind, chunk_size);
+#endif
+
       messageClose(LOG_STATS);
     }
 
     infoStreamPrint(LOG_STATS_V, 1, "function calls");
-    if (omc_flag[FLAG_DAE_MODE])
+    if (compiledInDAEMode)
     {
-      infoStreamPrint(LOG_STATS_V, 0, "%5ld calls of functionDAE", data->simulationInfo->callStatistics.functionEvalDAE);
+      infoStreamPrint(LOG_STATS_V, 1, "%5u calls of functionDAE", rt_ncall(SIM_TIMER_DAE));
+      infoStreamPrint(LOG_STATS_V, 0, "%12gs [%5.1f%%]", rt_accumulated(SIM_TIMER_DAE), rt_accumulated(SIM_TIMER_DAE)/total100);
+      messageClose(LOG_STATS_V);
     }
-    else
-    {
-      infoStreamPrint(LOG_STATS_V, 0, "%5ld calls of functionODE", data->simulationInfo->callStatistics.functionODE);
+    if (data->simulationInfo->callStatistics.functionODE) {
+      infoStreamPrint(LOG_STATS_V, 1, "%5ld calls of functionODE", data->simulationInfo->callStatistics.functionODE);
+      infoStreamPrint(LOG_STATS_V, 0, "%12gs [%5.1f%%]", rt_accumulated(SIM_TIMER_FUNCTION_ODE), rt_accumulated(SIM_TIMER_FUNCTION_ODE)/total100);
+      messageClose(LOG_STATS_V);
     }
-    infoStreamPrint(LOG_STATS_V, 0, "%5ld calls of functionAlgebraics", data->simulationInfo->callStatistics.functionAlgebraics);
+
+    if (rt_ncall(SIM_TIMER_RESIDUALS)) {
+      infoStreamPrint(LOG_STATS_V, 1, "%5d calls of functionODE_residual", rt_ncall(SIM_TIMER_RESIDUALS));
+      infoStreamPrint(LOG_STATS_V, 0, "%12gs [%5.1f%%]", rt_accumulated(SIM_TIMER_RESIDUALS), rt_accumulated(SIM_TIMER_RESIDUALS)/total100);
+      messageClose(LOG_STATS_V);
+    }
+
+    if (data->simulationInfo->callStatistics.functionAlgebraics) {
+      infoStreamPrint(LOG_STATS_V, 1, "%5ld calls of functionAlgebraics", data->simulationInfo->callStatistics.functionAlgebraics);
+      infoStreamPrint(LOG_STATS_V, 0, "%12gs [%5.1f%%]", rt_accumulated(SIM_TIMER_ALGEBRAICS), rt_accumulated(SIM_TIMER_ALGEBRAICS)/total100);
+      messageClose(LOG_STATS_V);
+    }
+
+    infoStreamPrint(LOG_STATS_V, 1, "%5d evaluations of jacobian", rt_ncall(SIM_TIMER_JACOBIAN));
+    infoStreamPrint(LOG_STATS_V, 0, "%12gs [%5.1f%%]", rt_accumulated(SIM_TIMER_JACOBIAN), rt_accumulated(SIM_TIMER_JACOBIAN)/total100);
+    messageClose(LOG_STATS_V);
+
     infoStreamPrint(LOG_STATS_V, 0, "%5ld calls of updateDiscreteSystem", data->simulationInfo->callStatistics.updateDiscreteSystem);
     infoStreamPrint(LOG_STATS_V, 0, "%5ld calls of functionZeroCrossingsEquations", data->simulationInfo->callStatistics.functionZeroCrossingsEquations);
-    infoStreamPrint(LOG_STATS_V, 0, "%5ld calls of functionZeroCrossings", data->simulationInfo->callStatistics.functionZeroCrossings);
+
+    infoStreamPrint(LOG_STATS_V, 1, "%5ld calls of functionZeroCrossings", data->simulationInfo->callStatistics.functionZeroCrossings);
+    infoStreamPrint(LOG_STATS_V, 0, "%12gs [%5.1f%%]", rt_accumulated(SIM_TIMER_ZC), rt_accumulated(SIM_TIMER_ZC)/total100);
+    messageClose(LOG_STATS_V);
+
     messageClose(LOG_STATS_V);
 
     infoStreamPrint(LOG_STATS_V, 1, "linear systems");
@@ -604,7 +684,7 @@ int finishSimulation(DATA* data, threadData_t *threadData, SOLVER_INFO* solverIn
 
     infoStreamPrint(LOG_STATS_V, 1, "non-linear systems");
     for(ui=0; ui<data->modelData->nNonLinearSystems; ui++)
-      printNonLinearSystemSolvingStatistics(data, ui, LOG_STATS_V);
+      printNonLinearSystemSolvingStatistics(&data->simulationInfo->nonlinearSystemData[ui], LOG_STATS_V);
     messageClose(LOG_STATS_V);
 
     messageClose(LOG_STATS);
@@ -652,13 +732,14 @@ int solver_main(DATA* data, threadData_t *threadData, const char* init_initMetho
     return 1;
 #endif
 
-#ifndef WITH_IPOPT
+#ifndef OMC_HAVE_IPOPT
   case S_OPTIMIZATION:
     warningStreamPrint(LOG_STDOUT, 0, "Ipopt is needed but not available.");
     TRACE_POP
     return 1;
 #endif
-
+  default:
+    break;
   }
 
   /* first initialize the model then allocate SolverData memory
@@ -666,12 +747,21 @@ int solver_main(DATA* data, threadData_t *threadData, const char* init_initMetho
    */
   simInfo->useStopTime = 1;
 
-  /* if the given step size is too small redefine it */
-  if ((simInfo->stepSize < MINIMAL_STEP_SIZE) && (simInfo->stopTime > 0)){
-    warningStreamPrint(LOG_STDOUT, 0, "The step-size %g is too small. Adjust the step-size to %g.", simInfo->stepSize, MINIMAL_STEP_SIZE);
-    simInfo->stepSize = MINIMAL_STEP_SIZE;
+  /* Use minStepSize if stepSize is getting too small, but
+   * allow stepSize to be zero if startTime == stopTime.
+   */
+  if ((simInfo->stepSize < simInfo->minStepSize) && (simInfo->stopTime > simInfo->startTime)){
+    warningStreamPrint(LOG_STDOUT, 0, "The step-size %g is too small. Adjust the step-size to %g.", simInfo->stepSize, simInfo->minStepSize);
+    simInfo->stepSize = simInfo->minStepSize;
     simInfo->numSteps = round((simInfo->stopTime - simInfo->startTime)/simInfo->stepSize);
   }
+  /* Check step size is not larger then stopTime-startTime, up to 6 decimals
+   * Ignored when linearizing model */
+  if (!data->modelData->create_linearmodel && simInfo->stepSize > (simInfo->stopTime - simInfo->startTime + 1e-7)) {
+    warningStreamPrint(LOG_STDOUT, 1, "Integrator step size greater than length of experiment");
+    infoStreamPrint(LOG_STDOUT, 0, "start time: %f, stop time: %f, integrator step size: %f",simInfo->startTime, simInfo->stopTime, simInfo->stepSize);
+    messageClose(LOG_STDOUT);
+  }
 #if !defined(OMC_EMCC)
     MMC_TRY_INTERNAL(simulationJumpBuffer)
 #endif
@@ -680,15 +770,19 @@ int solver_main(DATA* data, threadData_t *threadData, const char* init_initMetho
   externalInputallocate(data);
   /* set tolerance for ZeroCrossings */
   setZCtol(fmin(data->simulationInfo->stepSize, data->simulationInfo->tolerance));
-
   omc_alloc_interface.collect_a_little();
+
+  /* initialize solver data */
+  /* For the DAEmode we need to initialize solverData before the initialization,
+   * since the solver is used to obtain consistent values also via updateDiscreteSystem
+   */
+  retVal = initializeSolverData(data, threadData, &solverInfo);
+  initSolverInfo = 1;
+
   /* initialize all parts of the model */
-  retVal = initializeModel(data, threadData, init_initMethod, init_file, init_time);
-  omc_alloc_interface.collect_a_little();
-
-  if(0 == retVal) {
-    retVal = initializeSolverData(data, threadData, &solverInfo);
-    initSolverInfo = 1;
+  if (0 == retVal){
+    retVal = initializeModel(data, threadData, init_initMethod, init_file, init_time);
+    omc_alloc_interface.collect_a_little();
   }
 
 #if !defined(OMC_MINIMAL_RUNTIME)
@@ -789,7 +883,9 @@ int solver_main(DATA* data, threadData_t *threadData, const char* init_initMetho
   {
     freeSolverData(data, &solverInfo);
   }
-  infoStreamPrint(LOG_STDOUT, 0, "The simulation finished successfully.");
+
+  if (!retVal)
+    infoStreamPrint(LOG_SUCCESS, 0, "The simulation finished successfully.");
 
   TRACE_POP
   return retVal;
@@ -803,6 +899,8 @@ static int euler_ex_step(DATA* data, SOLVER_INFO* solverInfo)
   SIMULATION_DATA *sDataOld = (SIMULATION_DATA*)data->localData[1];
   modelica_real* stateDer = sDataOld->realVars + data->modelData->nStates;
 
+  if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
+
   solverInfo->currentTime = sDataOld->timeValue + solverInfo->currentStepSize;
 
   for(i = 0; i < data->modelData->nStates; i++)
@@ -817,6 +915,8 @@ static int euler_ex_step(DATA* data, SOLVER_INFO* solverInfo)
   /* function ODE evaluation is done directly after this function */
   solverInfo->solverStatsTmp[1] += 1;
 
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
+
   return 0;
 }
 
@@ -843,7 +943,7 @@ static int rungekutta_step_ssc(DATA* data, threadData_t *threadData, SOLVER_INFO
   modelica_real* stateDerOld = sDataOld->realVars + nx;
   double t = sDataOld->timeValue;
   const double targetTime = t + solverInfo->currentStepSize;
-  const short isMaxStepSizeSet = (short) omc_flagValue[FLAG_MAX_STEP_SIZE];
+  const short isMaxStepSizeSet = omc_flagValue[FLAG_MAX_STEP_SIZE] != NULL;
   const double maxStepSize = isMaxStepSizeSet ? atof(omc_flagValue[FLAG_MAX_STEP_SIZE]) : -1;
 #if defined(_MSC_VER)
   /* handle stupid compilers */
@@ -1018,6 +1118,8 @@ static int rungekutta_step(DATA* data, threadData_t *threadData, SOLVER_INFO* so
   modelica_real* stateDer = sData->realVars + data->modelData->nStates;
   modelica_real* stateDerOld = sDataOld->realVars + data->modelData->nStates;
 
+  if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
+
   solverInfo->currentTime = sDataOld->timeValue + solverInfo->currentStepSize;
 
   /* We calculate k[0] before returning from this function.
@@ -1031,11 +1133,13 @@ static int rungekutta_step(DATA* data, threadData_t *threadData, SOLVER_INFO* so
       sData->realVars[i] = sDataOld->realVars[i] + solverInfo->currentStepSize * rk->c[j] * k[j - 1][i];
     }
     sData->timeValue = sDataOld->timeValue + rk->c[j] * solverInfo->currentStepSize;
+    if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
     /* read input vars */
     externalInputUpdate(data);
     data->callback->input_function(data, threadData);
     /* eval ode equations */
     data->callback->functionODE(data, threadData);
+    if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
     memcpy(k[j], stateDer, data->modelData->nStates*sizeof(modelica_real));
 
   }
@@ -1056,12 +1160,13 @@ static int rungekutta_step(DATA* data, threadData_t *threadData, SOLVER_INFO* so
   solverInfo->solverStatsTmp[0] += 1;
   /* function ODE evaluation is done directly after this */
   solverInfo->solverStatsTmp[1] += rk->work_states_ndims+1;
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
 
   return 0;
 }
 
 /***************************************    Run Ipopt for optimization     ***********************************/
-#if defined(WITH_IPOPT)
+#if defined(OMC_HAVE_IPOPT)
 static int ipopt_step(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInfo)
 {
   int cJ, res;
