@@ -73,24 +73,6 @@
 
 namespace IAEX
 {
-  /*!
-   * \class SleeperThread
-   * \author Anders Ferström
-   *
-   * \brief Extends QThread. A small trick to get access to protected
-   * function in QThread.
-   */
-  class SleeperThread : public QThread
-  {
-  public:
-    static void msleep(unsigned long msecs)
-    {
-      QThread::msleep(msecs);
-    }
-  };
-
-
-
   /*!
    * \class MyTextEdit
    * \author Anders Ferström
@@ -491,8 +473,7 @@ namespace IAEX
     else
     {
       // 2006-01-30 AF, add message box
-      QString msg = "No Output style defened, please define a Output style in stylesheet.xml";
-      QMessageBox::warning( 0, "Warning", msg, "OK" );
+      QMessageBox::warning( 0, tr("Warning"), tr("No Output style defined, please define an Output style in stylesheet.xml"), "OK" );
     }
 
     QTextCursor cursor = output_->textCursor();
@@ -843,7 +824,7 @@ namespace IAEX
   QString InputCell::ChapterCounter()
   {
     if( chaptercounter_->toPlainText().isEmpty() )
-      return QString::null;
+      return QString();
 
     return chaptercounter_->toPlainText();
   }
@@ -858,7 +839,7 @@ namespace IAEX
   QString InputCell::ChapterCounterHtml()
   {
     if( chaptercounter_->toPlainText().isEmpty() )
-      return QString::null;
+      return QString();
 
     return chaptercounter_->toHtml();
   }
@@ -1092,9 +1073,8 @@ namespace IAEX
       // 2006-02-17 AF, set text '{evaluation expression}" during
       // evaluation of expressiuon
       output_->selectAll();
-      output_->textCursor().insertText( "{evaluating expression}" );
+      output_->textCursor().insertText( tr("{evaluating expression}") );
       setOutputStyle();
-      //output_->setPlainText( "{evaluating expression}" );
       output_->update();
       QCoreApplication::processEvents();
       delegate()->evalExpression(expr);
@@ -1165,7 +1145,6 @@ namespace IAEX
    */
   void InputCell::nextCommand()
   {
-    qDebug("Next Command");
     CommandCompletion *commandcompletion = CommandCompletion::instance( "commands.xml" );
     QTextCursor cursor = input_->textCursor();
 
@@ -1181,7 +1160,6 @@ namespace IAEX
    */
   void InputCell::nextField()
   {
-    qDebug("Next Field");
     CommandCompletion *commandcompletion = CommandCompletion::instance( "commands.xml" );
     QTextCursor cursor = input_->textCursor();
 
@@ -1254,14 +1232,6 @@ namespace IAEX
   }
 
 
-
-
-
-
-
-
-
-
   /*! \brief Do not use this member.
   *
   * This is an ugly part of the cell structure.
@@ -1277,7 +1247,6 @@ namespace IAEX
   void InputCell::removeCellWidgets()
   {
     /*
-    // PORT >> layout_->remove(input_);
     if(evaluated_)
       layout_->remove(output_);
       */
@@ -1296,7 +1265,6 @@ namespace IAEX
     {
       output_->clear();
       evaluated_ = false;
-      // PORT >> layout_->remove(output_);
       layout_->removeWidget(output_);
     }
 
@@ -1319,15 +1287,8 @@ namespace IAEX
     Cell::resizeEvent(event);
   }
 
-
-
-
-
-
-
   void InputCell::mouseDoubleClickEvent(QMouseEvent *)
   {
-    // PORT >>if(treeView()->hasMouse())
     if(treeView()->testAttribute(Qt::WA_UnderMouse))
     {
       setClosed(!closed_);
